//$Id: presentation.js,v 1.14 2008/10/18 18:25:04 silviogutierrez Exp $


//presentation Namespace
var presentation = presentation || {};
presentation.fn = presentation.fn || {};
presentation.settings = presentation.settings || {};
presentation.cache = presentation.cache || {};


$(document).ready(function() {  
  
  presentation.init();
  
  var pid = presentation.pid;
  
  // Finalize the loading process using the preloaded image.
  presentation.cycle.image = presentation.settings.currentData;
  presentation.finalize();

  if (presentation.settings.mode == 'grid' && pid) {
    presentation.switchModes();
    presentation.fetch(presentation.settings.fetchURL + pid, false, false);
    window.location.hash = "#" + pid;        
  }  
  else if (pid) {
    presentation.fetch(presentation.settings.fetchURL + pid, false, false);
  }
  else if (presentation.settings.mode == 'grid') {
    presentation.fetch(presentation.settings.fetchURL + presentation.initial, false, false);
    window.location.hash = "#";
  }
  else {
    presentation.fetch(presentation.settings.fetchURL + presentation.initial, false, true);
  }
 
  // As soon as we complete the first fetch, we have finished bootstrapping.
  presentation.bootstrap = true; 
 
});




/**
 * Bootstrap. Loads in settings, activates flags and switches into the 
 * correct mode. Any elements already in the DOM related to presentation
 * are activated.
 *
 */
presentation.init = function() {
  var path = window.location.href;

  var regex = /picture=([0-9]*)/;
  var match = path.match(regex);
  
  presentation.pid = match ? match[1] : 0;
  
  var fragment = path.split('#')[1]; // We want to know what pid the user provided, if any.  
  presentation.pid = (fragment == undefined) ? presentation.pid : fragment; // If undefined, ignore the pid.

  // Load settings.
  presentation.settings = Drupal.settings['presentation'];
  presentation.gridLoaded = false;
  presentation.mode = presentation.pid ? 'viewer' : presentation.settings.mode;
  presentation.initial = presentation.pid ? presentation.pid : presentation.settings.defaultImage;
  presentation.seed = '';
  presentation.bootstrap = false;
  
  // Initialize the flag system.
  presentation.resetFlags();
  
  // Initialize the cycle system.
  presentation.cycle = {};  
  
  $('.presentation-grid-link').click(function () {
    presentation.switchModes();

    return false;
  });
  
  
  presentation.activateLinks();
}





/**
 * Creates or resets buffering flags for each component.
 *
 */
presentation.resetFlags = function() {
  presentation.preparationFlags = {};
  presentation.callbackFlags = {};

  for (var component in presentation.settings.preparations) {
    presentation.preparationFlags[component] = false;
  }  
  
  for (var component in presentation.settings.callbacks) {
    presentation.callbackFlags[component] = false;
  }
  
  
}


/**
 * Flag a component indicating it's ready.
 *
 */
presentation.flag = function(component) {
  
  switch (presentation.state) {
    case 'preparation': {
      var callback = 'transition';      
      var flagType = 'preparationFlags';
      break;
    }
    case 'callback': {
      var callback = 'finalize';
      var flagType = 'callbackFlags';
      break;
    }
   
  
  }

  presentation[flagType][component] = true;
   
  if (presentation.checkFlags(flagType)) {
    presentation[callback]();
  }

}





/**
 * Checks all flags. If all flags are true, carry on with transition.
 *
 */
presentation.checkFlags = function(flagType) {
  
  for (var component in presentation[flagType]) {
    if (!presentation[flagType][component]) {
      return false;
    }
  }
  
  

  return true;
}



/**
 * Go through each link, add fading and fetching behavior.
 * This function can be called multiple times, since it will only
 * apply behaviors to new elements in the DOM. 
 *
 */
presentation.activateLinks = function() {
  $('.presentation-picture-link').each(function() {
      if (!$(this).is('.activated')) {
        $(this).click(function() {
          presentation.fetch(this.href);     
          return false;
        });
        
        var thumbnail = $(this).find('.presentation-thumbnail.translucent');
        
        $(thumbnail).css('opacity', .6);
        
        var mouseover = function () {
          $(thumbnail).css('opacity', 1);
          return true;
        }
        var mouseout = function () {
          $(thumbnail).css('opacity', .6);
          return true;
        }       

        $(thumbnail).hover(mouseover, mouseout);

        $(this).addClass('activated');
        
      }
      
      
  });  
}





/**
 * Flag the grid for reloading.
 *
 */
presentation.reloadGrid = function() {
  presentation.gridLoaded = false;
  
}




/**
 * This function switches between the viewer mode, and the grid mode.
 *
 * It keeps track of what mode presentation is in, and whether or not the grid
 * has ever been loaded yet. Calling reloadGrid() will force this method to
 * load the grid again.
 *
 */
presentation.switchModes = function() {
  if (!presentation.bootstrap) {
    presentation.mode = presentation.settings.mode;
  }
  presentation.mode = (presentation.mode == 'viewer') ? 'grid' : 'viewer';
  
  if ( false ){ //!presentation.gridLoaded) {
    $.get(presentation.settings.gridURL, null, function (data) {
      $('#presentation-grid').html(data);
      
      $('#presentation-viewer').hide();
      $('#presentation-grid').show();
      presentation.gridLoaded = true;
      
      
      
      if (presentation.mode == 'viewer') {
        $('#presentation-grid-link').html('All Photos');
        window.location.hash = "#" + (pid ? pid : "");
          
      }
      else {
        $('#presentation-grid-link').html('Single Photo');  
        window.location.hash = "";    
      }
        
      presentation.activateLinks();                    
    
    });
    
  }
  else {
    $('#presentation-viewer').toggle();
    $('#presentation-grid').toggle();
    
    if (presentation.mode == 'viewer') {
      $('#presentation-grid-link').html('All Photos');    
      window.location.hash = "#" + (presentation.pid ? presentation.pid : "");
        
    }
    else {    
      $('#presentation-grid-link').html('Single Picture');  
      window.location.hash = "#";    
    }
  }
  


}




/**
 * Preparation Callback.
 *
 * Fades out the main image to prepare it for replacement.
 *
 */
presentation.fn.hideImage = function() {

  // We do not wish to fade out since we are loading a cached picture.
  if (presentation.cache[presentation.cycle.pid] != undefined) {
    presentation.flag('presentation_frame');
    return;
  }
  
  $('#presentation-loader').css('background', 'url(' + presentation.settings.loaderImage + ') no-repeat center center');
  $('#presentation-display').fadeTo(200, 0.001, function () {
  
    presentation.flag('presentation_frame');   
  
  });
  

} 


/**
 * Callback.
 *
 * Fades in the main image after the AJAX request is done, and the buffer is loaded.
 *
 */
presentation.fn.showImage = function(image) {

  // We can assume the picture is already loaded if it's cached.
  if (presentation.cache[presentation.cycle.pid] != undefined) {
    _loadImage();
    return;
  }

  var imageBuffer = new Image(image['frameWidth'], image['frameHeight']);
  imageBuffer.onload = function() {
    _loadImage();
  };
  imageBuffer.src = image['framePath'] + presentation.seed;


  function _loadImage() {
    $('#presentation-loader').css('background-image', 'none');     
    $('#presentation-frame').attr({src: image.framePath + presentation.seed, width: image.frameWidth, height: image.frameHeight, title : image.caption});
    $('#presentation-display').fadeTo(200, 1, function() { 
      presentation.flag('presentation_frame');      
    });

  }

       
} 




/**
 * Callback.
 *
 * Updates all thumbnails once the AJAX request is done, and the buffer is loaded.
 *
 */
presentation.fn.updateThumbnails = function(image) {
  var thumbnails = image['thumbnails'];   
  
  // We can assume the thumbnails are loaded if they're cached.
  if (presentation.cache[presentation.cycle.pid] != undefined) {
    _updateThumbnails();
    return;
  }
  
  var buffer = new Array();
  var size = 0;
  
  for (var thumbnail in thumbnails) {
    size++;
  }
  
  
  
  //Boolean array holding flags.
  var flags = new Array(size);
  var count = 0;
  
  
  /** 
   * Creates a buffering system. Everytime an image is loaded, a flag is
   * set in an array. Once every flag is set, thumbnails are uploaded.
   *
   */
  for (var thumbnail in thumbnails) {
    buffer[count] = new Image(1, 1);
    buffer[count].flag = count;
    buffer[count].onload = function () {
      
      
      flags[this.flag] = true;
      
      if (_checkFlags()) {     
        _updateThumbnails();     
        return true;
      }
    
    }
    buffer[count].src = thumbnails[thumbnail].path + presentation.seed;
    
    count++;
  }
  
  /**
   * Helper function goes through each thumbnail and updates it.
   *
   */
  function _updateThumbnails() {
    for (var thumbnail in thumbnails) {
      $('#' + thumbnail).attr('href', thumbnails[thumbnail].link).find('img').attr('src', thumbnails[thumbnail].path + presentation.seed).attr('title', thumbnails[thumbnail].caption);
    }
    presentation.flag('presentation_links');        
  };
  
  /** 
   * Makes sure all thumbnails have been cleared.
   *
   */
  function _checkFlags() {
    for (var x = 0; x < size; x++) {
      if (flags[x] == undefined) {
        return false;
      }
    }
    
    return true;
  };  



}




presentation.refresh = function(refresh) {

  if (refresh == undefined) {
    refresh = true;
  }
  
  presentation.fetch(presentation.settings.fetchURL + presentation.pid, refresh);   
  
}



/**
 *  The main function to fetch a picture. 
 *  
 *  All picture switching should be done through this function.
 *  Before fetching, this function will fire preparation callbacks.
 *
 * @param location is a URL including a validated PID.
 * @param refresh is a boolean which will force the browser not to cache.
 * @param grid indicates this function should not switch to single view.
 *        Instead, the picture will load in the background.
 */
presentation.fetch = function(location, refresh, track) {
  
  var url = presentation.url(location);
  var pid = presentation.getPid(url);
  
  // Prepare the random seed if requested.
  presentation.seed = refresh ? '?rand=' + Math.random() : '';
  
  // Reset all flags.
  presentation.resetFlags();
  
  presentation.state = 'preparation';
  
  // Update the global cycle variables.
  presentation.cycle = {};
  presentation.cycle.url = url;
  presentation.cycle.refresh = !(refresh == false || refresh == undefined);
  presentation.cycle.track = (track == true || track == undefined);
  presentation.cycle.pid = pid;
  
  // Execute preparations for all components.
  for (prepare in presentation.settings.preparations) {
    presentation.fn[presentation.settings.preparations[prepare]]();
  }
  
  //presentation.current = presentation.pid;
  
}



presentation.transition = function() {   
  
  
  
  var _transition = function(image) {
    presentation.state = 'callback';
    presentation.cycle.image = image;
    
    for (callback in presentation.settings.callbacks) {
      presentation.fn[presentation.settings.callbacks[callback]](image);
    }    
  }
  
  
  // We only need to make a request if it hasn't been made already, or if 
  // refresh is marked as true.
  if (presentation.cache[presentation.cycle.pid] != undefined && !presentation.cycle.refresh) {
    _transition(presentation.cache[presentation.cycle.pid]);
  }
  else {
    $.get(presentation.cycle.url, null, function(data) {
      var image = presentation.parseJson(data);
      
      if (!image['success']) {
        presentation.setError(image['message']);
      }
      else {
        _transition(image);
      }
    });
  }
}


presentation.finalize = function() {

  var image = presentation.cycle.image;
  
  //Reset the message box.
  presentation.clearMessage();
  
  if (presentation.cycle.track) {
    window.location.hash = '#' + image['key'];    
  }
  
  presentation.pid = image['key'];
  
  // Store the request in a static cache.
  presentation.cache[image['key']] = image;
    
}




/**************************** Utilities *******************************/

/**
 * Converts a regular HTML address to an AJAX one.
 *
 */
presentation.url = function(incoming) {

  var regex = /picture=([0-9]*)/;
  
  if (regex.test(incoming)) {
    var match = incoming.match(regex)
    var url = presentation.settings.fetchURL + match[1];
  }
  else {
    var url = incoming;
  }
 
  return url;
}

/**
 * Get the PID of a picture from a URL.
 *
 */
presentation.getPid = function(url) {
  var index = url.lastIndexOf('/');
  return url.substring(index + 1);
  
}


/**
 * Clears the local cache. If a pid is given, only that picture will be
 * deleted from the cache. Otherwise, the entire cache is deleted.
 *
 */
presentation.clearCache = function(pid) {
  if (pid != undefined) {
    delete presentation.cache[pid];
  }
  else {
    presentation.cache = {};
  }
}


